@@ -4,16 +4,19 @@ import importlib as _il
 import os as _os
 import typing as _typing
 
-from . import references as _references
-from .finite_element import FiniteElement as _FiniteElement
-from .geometry import SetOfPointsInput as _SetOfPointsInput
-from .geometry import parse_set_of_points_input as _parse_set_of_points_input
+import symfem.references as _references
+from symfem.finite_element import FiniteElement as _FiniteElement
+from symfem.finite_element import CiarletElement as _CiarletElement
+from symfem.geometry import SetOfPointsInput as _SetOfPointsInput
+from symfem.geometry import parse_set_of_points_input as _parse_set_of_points_input
 
 _folder = _os.path.dirname(_os.path.realpath(__file__))
 
 _elementmap: _typing.Dict[str, _typing.Dict[str, _typing.Type]] = {}
 _elementlist: _typing.List[_typing.Type] = []
 
+__all__ = ["add_element", "create_reference", "create_element"]
+
 
 def add_element(element_class: _typing.Type):
     """Add an element to Symfem.
@@ -60,7 +63,7 @@ for _file in _os.listdir(_os.path.join(_folder, "elements")):
 
 
 def create_reference(
-    cell_type: str, vertices: _SetOfPointsInput = None
+    cell_type: str, vertices: _typing.Optional[_SetOfPointsInput] = None
 ) -> _references.Reference:
     """Make a reference cell.
 
@@ -98,7 +101,11 @@ def create_reference(
 
 
 def create_element(
-    cell_type: str, element_type: str, order: int, **kwargs: _typing.Any
+    cell_type: str,
+    element_type: str,
+    order: int,
+    vertices: _typing.Optional[_SetOfPointsInput] = None,
+    **kwargs: _typing.Any,
 ) -> _FiniteElement:
     """Make a finite element.
 
@@ -155,7 +162,8 @@ def create_element(
                       Bernardi-Raugel,
                       Wu-Xu,
                       transition,
-                      Guzman-Neilan,
+                      Guzman-Neilan first kind, Guzman-Neilan,
+                      Guzman-Neilan second kind,
                       nonconforming Arnold-Winther, nonconforming AW,
                       TScurl, trimmed serendipity Hcurl,
                       TSdiv, trimmed serendipity Hdiv,
@@ -165,10 +173,22 @@ def create_element(
                       Arnold-Boffi-Falk, ABF,
                       Arbogast-Correa, AC, AC full, Arbogast-Correa full,
                       Rannacher-Turek,
-                      P1-iso-P2, P2-iso-P1, iso-P2 P1
+                      P1-iso-P2, P2-iso-P1, iso-P2 P1,
+                      Huang-Zhang, HZ,
+                      enriched Galerkin, EG,
+                      enriched vector Galerkin, locking-free enriched Galerkin, LFEG,
+                      P1 macro,
+                      Alfeld-Sorokina, AS,
+                      Gopalakrishnan-Lederer-Schoberl, GLS
         order: The order of the element.
+        vertices: The vertices of the reference.
     """
-    reference = create_reference(cell_type)
+    reference = create_reference(cell_type, vertices=vertices)
+
+    if reference.tdim != reference.gdim:
+        raise ValueError(
+            "Cannot create element on cell with different topological and geometric dimensions."
+        )
 
     if element_type in _elementmap:
         if reference.name not in _elementmap[element_type]:
@@ -178,12 +198,20 @@ def create_element(
             raise ValueError(f"Order {order} {element_type} element cannot be created.")
         return element_class(reference, order, **kwargs)
 
+    if element_type.startswith("discontinuous "):
+        e = create_element(cell_type, element_type[14:], order, vertices, **kwargs)
+        if isinstance(e, _CiarletElement):
+            for d in e.dofs:
+                d.entity = (reference.tdim, 0)
+            e.continuity = "L2"
+            e.names = ["discontinuous " + i for i in e.names]
+
+            return e
+
     raise ValueError(f"Unsupported element type: {element_type}")
 
 
-def _order_is_allowed(
-    element_class: _typing.Type, ref: str, order: int
-) -> bool:
+def _order_is_allowed(element_class: _typing.Type, ref: str, order: int) -> bool:
     """Check that an order is valid for an element.
 
     Args:
