@@ -9,12 +9,13 @@ from itertools import product
 
 import sympy
 
-from ..finite_element import CiarletElement
-from ..functionals import DotPointEvaluation, ListOfFunctionals, PointEvaluation
-from ..functions import FunctionInput
-from ..polynomials import polynomial_set_1d, quolynomial_set_1d
-from ..references import Reference
-from .lagrange import Lagrange
+from symfem.elements.lagrange import Lagrange
+from symfem.finite_element import CiarletElement
+from symfem.functionals import DotPointEvaluation, ListOfFunctionals, PointEvaluation
+from symfem.functions import FunctionInput
+from symfem.references import Reference
+
+__all__ = ["Bubble", "BubbleEnrichedLagrange", "BubbleEnrichedVectorLagrange"]
 
 
 class Bubble(CiarletElement):
@@ -28,43 +29,54 @@ class Bubble(CiarletElement):
             order: The polynomial order
             variant: The variant of the element
         """
-        from .. import create_element
-        p1 = create_element(reference.name, "Lagrange", 1)
+        from symfem import create_element
+
+        p1 = create_element(reference.name, "Lagrange", 1, vertices=reference.vertices)
+        basis = p1.get_basis_functions()
         bubble = 1
-        for f in p1.get_basis_functions():
-            bubble *= f
+        if reference.name in "quadrilateral":
+            bubble *= basis[0]
+            bubble *= basis[3]
+        elif reference.name in "hexahedron":
+            bubble *= basis[0]
+            bubble *= basis[7]
+        else:
+            for f in basis:
+                bubble *= f
 
-        poly: typing.List[FunctionInput] = []
-
-        # TODO: variants
-        if reference.name == "interval":
-            poly += [bubble * p for p in polynomial_set_1d(reference.tdim, order - 2)]
+        if reference.name in ["interval", "quadrilateral", "hexahedron"]:
+            o = order - 2
         elif reference.name == "triangle":
-            poly += [bubble * p for p in polynomial_set_1d(reference.tdim, order - 3)]
+            o = order - 3
         elif reference.name == "tetrahedron":
-            poly += [bubble * p for p in polynomial_set_1d(reference.tdim, order - 4)]
-        elif reference.name == "quadrilateral":
-            poly += [bubble * p for p in quolynomial_set_1d(reference.tdim, order - 2)]
-        elif reference.name == "hexahedron":
-            poly += [bubble * p for p in quolynomial_set_1d(reference.tdim, order - 2)]
+            o = order - 4
         else:
             raise ValueError(f"Unsupported reference: {reference.name}")
 
+        pn = create_element(
+            reference.name, "Lagrange", o, vertices=reference.vertices, variant=variant
+        )
+
+        poly = [bubble * p for p in pn.get_basis_functions()]
+
         dofs: ListOfFunctionals = []
         if reference.name in ["interval", "triangle", "tetrahedron"]:
-            def func(i): return sum(i)
+
+            def func(i):
+                return sum(i)
         else:
-            def func(i): return max(i)
+
+            def func(i):
+                return max(i)
+
         for i in product(range(1, order), repeat=reference.tdim):
             if func(i) < order:
-                point = tuple(sympy.Rational(j, order) for j in i)
+                point = reference.get_point(tuple(sympy.Rational(j, order) for j in i[::-1]))
                 dofs.append(PointEvaluation(reference, point, entity=(reference.tdim, 0)))
 
         self.variant = variant
 
-        super().__init__(
-            reference, order, poly, dofs, reference.tdim, 1
-        )
+        super().__init__(reference, order, poly, dofs, reference.tdim, 1)
 
     def init_kwargs(self) -> typing.Dict[str, typing.Any]:
         """Return the kwargs used to create this element.
@@ -74,11 +86,36 @@ class Bubble(CiarletElement):
         """
         return {"variant": self.variant}
 
+    @property
+    def lagrange_subdegree(self) -> int:
+        return -1
+
+    @property
+    def lagrange_superdegree(self) -> typing.Optional[int]:
+        return self.order
+
+    @property
+    def polynomial_subdegree(self) -> int:
+        return -1
+
+    @property
+    def polynomial_superdegree(self) -> typing.Optional[int]:
+        if self.reference.name in ["quadrilateral", "hexahedron"]:
+            return self.order * self.reference.tdim
+        return self.order
+
     names = ["bubble"]
     references = ["interval", "triangle", "tetrahedron", "quadrilateral", "hexahedron"]
-    min_order = {"interval": 2, "triangle": 3, "tetrahedron": 4,
-                 "quadrilateral": 2, "hexahedron": 2}
+    min_order = {
+        "interval": 2,
+        "triangle": 3,
+        "tetrahedron": 4,
+        "quadrilateral": 2,
+        "hexahedron": 2,
+    }
     continuity = "C0"
+    value_type = "scalar"
+    last_updated = "2025.10"
 
 
 class BubbleEnrichedLagrange(CiarletElement):
@@ -102,10 +139,7 @@ class BubbleEnrichedLagrange(CiarletElement):
 
         self.variant = variant
 
-        super().__init__(
-            reference, order, poly,
-            lagrange.dofs + bubble.dofs, reference.tdim, 1
-        )
+        super().__init__(reference, order, poly, lagrange.dofs + bubble.dofs, reference.tdim, 1)
 
     def init_kwargs(self) -> typing.Dict[str, typing.Any]:
         """Return the kwargs used to create this element.
@@ -115,10 +149,28 @@ class BubbleEnrichedLagrange(CiarletElement):
         """
         return {"variant": self.variant}
 
+    @property
+    def lagrange_subdegree(self) -> int:
+        return self.order
+
+    @property
+    def lagrange_superdegree(self) -> typing.Optional[int]:
+        return self.order + 2
+
+    @property
+    def polynomial_subdegree(self) -> int:
+        return self.order
+
+    @property
+    def polynomial_superdegree(self) -> typing.Optional[int]:
+        return self.order + 2
+
     names = ["bubble enriched Lagrange"]
     references = ["triangle"]
     min_order = 1
     continuity = "C0"
+    value_type = "scalar"
+    last_updated = "2024.09"
 
 
 class BubbleEnrichedVectorLagrange(CiarletElement):
@@ -141,9 +193,11 @@ class BubbleEnrichedVectorLagrange(CiarletElement):
                 poly.append((p, 0))
                 poly.append((0, p))
 
-        dofs: ListOfFunctionals = [DotPointEvaluation(reference, d.dof_point(), v, entity=d.entity)
-                                   for d in lagrange.dofs + bubble.dofs
-                                   for v in [(1, 0), (0, 1)]]
+        dofs: ListOfFunctionals = [
+            DotPointEvaluation(reference, d.dof_point(), v, entity=d.entity)
+            for d in lagrange.dofs + bubble.dofs
+            for v in [(1, 0), (0, 1)]
+        ]
 
         self.variant = variant
 
@@ -157,7 +211,25 @@ class BubbleEnrichedVectorLagrange(CiarletElement):
         """
         return {"variant": self.variant}
 
+    @property
+    def lagrange_subdegree(self) -> int:
+        return self.order
+
+    @property
+    def lagrange_superdegree(self) -> typing.Optional[int]:
+        return self.order + 2
+
+    @property
+    def polynomial_subdegree(self) -> int:
+        return self.order
+
+    @property
+    def polynomial_superdegree(self) -> typing.Optional[int]:
+        return self.order + 2
+
     names = ["bubble enriched vector Lagrange"]
     references = ["triangle"]
     min_order = 1
     continuity = "C0"
+    value_type = "vector"
+    last_updated = "2024.09"
