@@ -4,10 +4,14 @@ This element's definition appears in https://arxiv.org/abs/1809.02192
 (Arbogast, Tao, 2018)
 """
 
-from ..finite_element import DirectElement
-from ..references import Reference
-from ..symbols import x
-from .dpc import DPC
+import typing
+
+from symfem.elements.dpc import DPC
+from symfem.finite_element import DirectElement
+from symfem.references import Reference
+from symfem.symbols import x
+
+__all__ = ["DirectSerendipity"]
 
 
 class DirectSerendipity(DirectElement):
@@ -50,19 +54,19 @@ class DirectSerendipity(DirectElement):
             lambda_12 = xi_v * (1 - x[0]) + eta_v * x[0]
 
             for j in range(order - 1):
-                basis_functions.append((1 - x[1]) * x[1] * lambda_h ** j)
+                basis_functions.append((1 - x[1]) * x[1] * lambda_h**j)
                 basis_entities.append((1, 1))
             for j in range(order - 2):
-                basis_functions.append((1 - x[1]) * x[1] * lambda_12 * lambda_h ** j)
+                basis_functions.append((1 - x[1]) * x[1] * lambda_12 * lambda_h**j)
                 basis_entities.append((1, 2))
             basis_functions.append((1 - x[1]) * x[1] * r_v * lambda_h ** (order - 2))
             basis_entities.append((1, 2))
 
             for j in range(order - 1):
-                basis_functions.append((1 - x[0]) * x[0] * lambda_v ** j)
+                basis_functions.append((1 - x[0]) * x[0] * lambda_v**j)
                 basis_entities.append((1, 0))
             for j in range(order - 2):
-                basis_functions.append((1 - x[0]) * x[0] * lambda_34 * lambda_v ** j)
+                basis_functions.append((1 - x[0]) * x[0] * lambda_34 * lambda_v**j)
                 basis_entities.append((1, 3))
             basis_functions.append((1 - x[0]) * x[0] * r_h * lambda_v ** (order - 2))
             basis_entities.append((1, 3))
@@ -73,10 +77,27 @@ class DirectSerendipity(DirectElement):
                 basis_functions.append(f * x[0] * x[1] * (1 - x[0]) * (1 - x[1]))
                 basis_entities.append((2, 0))
 
-        super().__init__(reference, order, basis_functions, basis_entities,
-                         reference.tdim, 1)
+        super().__init__(reference, order, basis_functions, basis_entities, reference.tdim, 1)
+
+    @property
+    def lagrange_subdegree(self) -> int:
+        raise NotImplementedError()
+
+    @property
+    def lagrange_superdegree(self) -> typing.Optional[int]:
+        raise NotImplementedError()
+
+    @property
+    def polynomial_subdegree(self) -> int:
+        raise NotImplementedError()
+
+    @property
+    def polynomial_superdegree(self) -> typing.Optional[int]:
+        raise NotImplementedError()
 
     names = ["direct serendipity"]
     references = ["quadrilateral"]
     min_order = 1
     continuity = "C0"
+    value_type = "scalar non-polynomial"
+    last_updated = "2023.05"
