@@ -6,11 +6,13 @@ This element's definition appears in https://doi.org/10.1016/0045-7825(72)90006-
 
 import typing
 
-from ..finite_element import CiarletElement
-from ..functionals import DerivativePointEvaluation, ListOfFunctionals, PointEvaluation
-from ..functions import FunctionInput
-from ..polynomials import polynomial_set_1d
-from ..references import Reference
+from symfem.finite_element import CiarletElement
+from symfem.functionals import DerivativePointEvaluation, ListOfFunctionals, PointEvaluation
+from symfem.functions import FunctionInput
+from symfem.polynomials import polynomial_set_1d
+from symfem.references import Reference
+
+__all__ = ["Hermite"]
 
 
 class Hermite(CiarletElement):
@@ -28,9 +30,14 @@ class Hermite(CiarletElement):
         for v_n, v in enumerate(reference.vertices):
             dofs.append(PointEvaluation(reference, v, entity=(0, v_n)))
             for i in range(reference.tdim):
-                dofs.append(DerivativePointEvaluation(
-                    reference, v, tuple(1 if i == j else 0 for j in range(reference.tdim)),
-                    entity=(0, v_n)))
+                dofs.append(
+                    DerivativePointEvaluation(
+                        reference,
+                        v,
+                        tuple(1 if i == j else 0 for j in range(reference.tdim)),
+                        entity=(0, v_n),
+                    )
+                )
         for e_n in range(reference.sub_entity_count(2)):
             sub_entity = reference.sub_entity(2, e_n)
             dofs.append(PointEvaluation(reference, sub_entity.midpoint(), entity=(2, e_n)))
@@ -40,8 +47,26 @@ class Hermite(CiarletElement):
 
         super().__init__(reference, order, poly, dofs, reference.tdim, 1)
 
+    @property
+    def lagrange_subdegree(self) -> int:
+        return self.order
+
+    @property
+    def lagrange_superdegree(self) -> typing.Optional[int]:
+        return self.order
+
+    @property
+    def polynomial_subdegree(self) -> int:
+        return self.order
+
+    @property
+    def polynomial_superdegree(self) -> typing.Optional[int]:
+        return self.order
+
     names = ["Hermite"]
     references = ["interval", "triangle", "tetrahedron"]
     min_order = 3
     max_order = 3
     continuity = "C1"
+    value_type = "scalar"
+    last_updated = "2023.05"
