@@ -5,17 +5,46 @@ from abc import ABC, abstractmethod
 
 import sympy
 
-from .functions import AnyFunction, VectorFunction
-from .geometry import (PointType, PointTypeInput, SetOfPoints, SetOfPointsInput, parse_point_input,
-                       parse_set_of_points_input)
+from symfem.functions import Function, VectorFunction
+from symfem.geometry import (
+    PointType,
+    PointTypeInput,
+    SetOfPoints,
+    SetOfPointsInput,
+    parse_point_input,
+    parse_set_of_points_input,
+)
 
-PointOrFunction = typing.Union[PointTypeInput, AnyFunction]
+__all__ = [
+    "PointOrFunction",
+    "SetOfPointsOrFunctions",
+    "tex_font_size",
+    "Colors",
+    "PictureElement",
+    "Line",
+    "Bezier",
+    "Arrow",
+    "NCircle",
+    "Fill",
+    "Math",
+    "Picture",
+]
+
+PointOrFunction = typing.Union[PointTypeInput, Function]
 SetOfPointsOrFunctions = typing.Union[
-    typing.List[PointOrFunction], typing.Tuple[PointOrFunction, ...]]
+    typing.List[PointOrFunction], typing.Tuple[PointOrFunction, ...]
+]
 
 
 def tex_font_size(n: int):
-    """Convert a font size to a tex size command."""
+    """Convert a font size to a TeX size command.
+
+    Args:
+        n: Font size
+
+    Returns:
+        TeX size command
+    """
     if n < 21:
         return "\\tiny"
     if n < 24:
@@ -47,6 +76,7 @@ class Colors:
     GREEN = "#55FF00"
     PURPLE = "#DD2299"
     GRAY = "#AAAAAA"
+    RED = "#FF0000"
 
     def __init__(self):
         """Initialise."""
@@ -72,7 +102,14 @@ class Colors:
         raise ValueError(f"Unsupported dimension: {n}")
 
     def get_tikz_name(self, name: str) -> str:
-        """Get the name of the colour to be used in Tikz."""
+        """Get the name of the colour to be used in Tikz.
+
+        Args:
+            name: HTML name of the color
+
+        Returns:
+            The Tikz name of the color
+        """
         if name.startswith("#"):
             if name not in self._tikz:
                 self._tikz[name] = f"customcolor{len(self._tikz)}"
@@ -80,7 +117,11 @@ class Colors:
         return name
 
     def get_tikz_definitions(self) -> str:
-        """Get the definitions of colours used in Tikz."""
+        """Get the definitions of colours used in Tikz.
+
+        Returns:
+            Definitions of Tikz colors
+        """
         out = ""
         for a, b in self._tikz.items():
             assert a.startswith("#")
@@ -88,20 +129,16 @@ class Colors:
         return out
 
 
-colors = Colors()
-
-
 class PictureElement(ABC):
     """An element in a picture."""
 
-    def __init__(self):
+    def __init__(self, colors: Colors, midpoint: typing.Tuple[float, ...] = ()):
         """Create an element."""
-        pass
+        self.midpoint = midpoint
+        self.colors = colors
 
     @abstractmethod
-    def as_svg(
-        self, map_pt: typing.Callable[[PointType], typing.Tuple[float, float]]
-    ) -> str:
+    def as_svg(self, map_pt: typing.Callable[[PointType], typing.Tuple[float, float]]) -> str:
         """Return SVG format.
 
         Args:
@@ -113,9 +150,7 @@ class PictureElement(ABC):
         pass
 
     @abstractmethod
-    def as_tikz(
-        self, map_pt: typing.Callable[[PointType], typing.Tuple[float, float]]
-    ) -> str:
+    def as_tikz(self, map_pt: typing.Callable[[PointType], typing.Tuple[float, float]]) -> str:
         """Return Tikz format.
 
         Args:
@@ -172,27 +207,23 @@ class PictureElement(ABC):
 class Line(PictureElement):
     """A line."""
 
-    def __init__(
-        self, start: PointType, end: PointType, color: str,
-        width: float
-    ):
+    def __init__(self, colors: Colors, start: PointType, end: PointType, color: str, width: float):
         """Create a line.
 
         Args:
+            colors: Colors object for the current picture
             start: The start point
             end: The end point
             color: The color
             width: The width of the line
         """
-        super().__init__()
+        super().__init__(colors, tuple(sum(i) for i in zip(start, end)))
         self.start = start
         self.end = end
         self.color = color
         self.width = width
 
-    def as_svg(
-        self, map_pt: typing.Callable[[PointType], typing.Tuple[float, float]]
-    ) -> str:
+    def as_svg(self, map_pt: typing.Callable[[PointType], typing.Tuple[float, float]]) -> str:
         """Return SVG format.
 
         Args:
@@ -203,12 +234,12 @@ class Line(PictureElement):
         """
         s = map_pt(self.start)
         e = map_pt(self.end)
-        return (f"<line x1='{s[0]}' y1='{s[1]}' x2='{e[0]}' y2='{e[1]}' "
-                f"stroke='{self.color}' stroke-width='{self.width}' stroke-linecap='round' />\n")
+        return (
+            f"<line x1='{s[0]}' y1='{s[1]}' x2='{e[0]}' y2='{e[1]}' "
+            f"stroke='{self.color}' stroke-width='{self.width}' stroke-linecap='round' />\n"
+        )
 
-    def as_tikz(
-        self, map_pt: typing.Callable[[PointType], typing.Tuple[float, float]]
-    ) -> str:
+    def as_tikz(self, map_pt: typing.Callable[[PointType], typing.Tuple[float, float]]) -> str:
         """Return Tikz format.
 
         Args:
@@ -219,8 +250,10 @@ class Line(PictureElement):
         """
         s = map_pt(self.start)
         e = map_pt(self.end)
-        return (f"\\draw[{colors.get_tikz_name(self.color)},line width={self.width * 0.2}pt,"
-                f"line cap=round] ({s[0]},{s[1]}) -- ({e[0]},{e[1]});\n")
+        return (
+            f"\\draw[{self.colors.get_tikz_name(self.color)},line width={self.width * 0.2}pt,"
+            f"line cap=round] ({s[0]},{s[1]}) -- ({e[0]},{e[1]});\n"
+        )
 
     @property
     def points(self) -> SetOfPoints:
@@ -236,12 +269,19 @@ class Bezier(PictureElement):
     """A Bezier curve."""
 
     def __init__(
-        self, start: PointType, mid1: PointType, mid2: PointType, end: PointType, color: str,
-        width: float
+        self,
+        colors: Colors,
+        start: PointType,
+        mid1: PointType,
+        mid2: PointType,
+        end: PointType,
+        color: str,
+        width: float,
     ):
         """Create a Bezier curve.
 
         Args:
+            colors: Colors object for the current picture
             start: The start point
             mid1: The first control point
             mid2: The second control point
@@ -249,7 +289,7 @@ class Bezier(PictureElement):
             color: The color
             width: The width of the line
         """
-        super().__init__()
+        super().__init__(colors, tuple(sum(i) for i in zip(start, end)))
         self.start = start
         self.mid1 = mid1
         self.mid2 = mid2
@@ -257,9 +297,7 @@ class Bezier(PictureElement):
         self.color = color
         self.width = width
 
-    def as_svg(
-        self, map_pt: typing.Callable[[PointType], typing.Tuple[float, float]]
-    ) -> str:
+    def as_svg(self, map_pt: typing.Callable[[PointType], typing.Tuple[float, float]]) -> str:
         """Return SVG format.
 
         Args:
@@ -272,13 +310,13 @@ class Bezier(PictureElement):
         m1 = map_pt(self.mid1)
         m2 = map_pt(self.mid2)
         e = map_pt(self.end)
-        return (f"<path d='M{s[0]},{s[1]} C{m1[0]},{m1[1]}, {m2[0]},{m2[1]}, {e[0]},{e[1]}' "
-                f"stroke='{self.color}' stroke-width='{self.width}' stroke-linecap='round' "
-                " fill='none' />\n")
+        return (
+            f"<path d='M{s[0]},{s[1]} C{m1[0]},{m1[1]}, {m2[0]},{m2[1]}, {e[0]},{e[1]}' "
+            f"stroke='{self.color}' stroke-width='{self.width}' stroke-linecap='round' "
+            " fill='none' />\n"
+        )
 
-    def as_tikz(
-        self, map_pt: typing.Callable[[PointType], typing.Tuple[float, float]]
-    ) -> str:
+    def as_tikz(self, map_pt: typing.Callable[[PointType], typing.Tuple[float, float]]) -> str:
         """Return Tikz format.
 
         Args:
@@ -291,9 +329,11 @@ class Bezier(PictureElement):
         m1 = map_pt(self.mid1)
         m2 = map_pt(self.mid2)
         e = map_pt(self.end)
-        return (f"\\draw[{colors.get_tikz_name(self.color)},line width={self.width * 0.2}pt,"
-                f"line cap=round] ({s[0]},{s[1]}) .. controls ({m1[0]},{m1[1]}) "
-                f"and ({m2[0]},{m2[1]}) .. ({e[0]},{e[1]});\n")
+        return (
+            f"\\draw[{self.colors.get_tikz_name(self.color)},line width={self.width * 0.2}pt,"
+            f"line cap=round] ({s[0]},{s[1]}) .. controls ({m1[0]},{m1[1]}) "
+            f"and ({m2[0]},{m2[1]}) .. ({e[0]},{e[1]});\n"
+        )
 
     @property
     def points(self) -> SetOfPoints:
@@ -308,27 +348,23 @@ class Bezier(PictureElement):
 class Arrow(PictureElement):
     """An arrow."""
 
-    def __init__(
-        self, start: PointType, end: PointType, color: str,
-        width: float
-    ):
+    def __init__(self, colors: Colors, start: PointType, end: PointType, color: str, width: float):
         """Create an arrow.
 
         Args:
+            colors: Colors object for the current picture
             start: The start point
             end: The end point
             color: The color
             width: The width of the line
         """
-        super().__init__()
+        super().__init__(colors, tuple(sum(i) for i in zip(start, end)))
         self.start = start
         self.end = end
         self.color = color
         self.width = width
 
-    def as_svg(
-        self, map_pt: typing.Callable[[PointType], typing.Tuple[float, float]]
-    ) -> str:
+    def as_svg(self, map_pt: typing.Callable[[PointType], typing.Tuple[float, float]]) -> str:
         """Return SVG format.
 
         Args:
@@ -341,8 +377,10 @@ class Arrow(PictureElement):
 
         s = map_pt(self.start)
         e = map_pt(self.end)
-        out += (f"<line x1='{s[0]}' y1='{s[1]}' x2='{e[0]}' y2='{e[1]}' "
-                f"stroke='{self.color}' stroke-width='{self.width}' stroke-linecap='round' />\n")
+        out += (
+            f"<line x1='{s[0]}' y1='{s[1]}' x2='{e[0]}' y2='{e[1]}' "
+            f"stroke='{self.color}' stroke-width='{self.width}' stroke-linecap='round' />\n"
+        )
 
         ve = VectorFunction(self.end)
         vs = VectorFunction(self.start)
@@ -356,14 +394,14 @@ class Arrow(PictureElement):
             pt_s = pt.as_sympy()
             assert isinstance(pt_s, tuple)
             m = map_pt(pt_s)
-            out += (f"<line x1='{m[0]}' y1='{m[1]}' x2='{e[0]}' y2='{e[1]}' "
-                    f"stroke='{self.color}' stroke-width='{self.width}' stroke-linecap='round'"
-                    " />\n")
+            out += (
+                f"<line x1='{m[0]}' y1='{m[1]}' x2='{e[0]}' y2='{e[1]}' "
+                f"stroke='{self.color}' stroke-width='{self.width}' stroke-linecap='round'"
+                " />\n"
+            )
         return out
 
-    def as_tikz(
-        self, map_pt: typing.Callable[[PointType], typing.Tuple[float, float]]
-    ) -> str:
+    def as_tikz(self, map_pt: typing.Callable[[PointType], typing.Tuple[float, float]]) -> str:
         """Return Tikz format.
 
         Args:
@@ -374,9 +412,11 @@ class Arrow(PictureElement):
         """
         s = map_pt(self.start)
         e = map_pt(self.end)
-        return (f"\\draw[-stealth,{colors.get_tikz_name(self.color)},"
-                f"line width={self.width * 0.2}pt,line cap=round] "
-                f"({s[0]},{s[1]}) -- ({e[0]},{e[1]});\n")
+        return (
+            f"\\draw[-stealth,{self.colors.get_tikz_name(self.color)},"
+            f"line width={self.width * 0.2}pt,line cap=round] "
+            f"({s[0]},{s[1]}) -- ({e[0]},{e[1]});\n"
+        )
 
     @property
     def points(self) -> SetOfPoints:
@@ -392,12 +432,22 @@ class NCircle(PictureElement):
     """A circle containing a number."""
 
     def __init__(
-        self, centre: PointType, number: int, color: str, text_color: str, fill_color: str,
-        radius: float, font_size: typing.Union[int, None], width: float, font: str
+        self,
+        colors: Colors,
+        centre: PointType,
+        number: int,
+        color: str,
+        text_color: str,
+        fill_color: str,
+        radius: float,
+        font_size: typing.Union[int, None],
+        width: float,
+        font: str,
     ):
         """Create a circle containing a number.
 
         Args:
+            colors: Colors object for the current picture
             centre: The centre of the circle
             number: The number to print inside the circle
             color: The colour of the circle's line
@@ -408,6 +458,7 @@ class NCircle(PictureElement):
             width: The width of the line
             font: The font to use for the number
         """
+        super().__init__(colors, centre)
         self.centre = centre
         self.number = number
         self.color = color
@@ -426,9 +477,7 @@ class NCircle(PictureElement):
             self.font_size = font_size
         self.width = width
 
-    def as_svg(
-        self, map_pt: typing.Callable[[PointType], typing.Tuple[float, float]]
-    ) -> str:
+    def as_svg(self, map_pt: typing.Callable[[PointType], typing.Tuple[float, float]]) -> str:
         """Return SVG format.
 
         Args:
@@ -439,15 +488,15 @@ class NCircle(PictureElement):
         """
         c = map_pt(self.centre)
 
-        return (f"<circle cx='{c[0]}' cy='{c[1]}' r='{self.radius}' "
-                f"stroke='{self.color}' stroke-width='{self.width}' fill='{self.fill_color}' />\n"
-                f"<text x='{c[0]}' y='{c[1]}' fill='{self.text_color}' font-size='{self.font_size}'"
-                f" style=\"text-anchor:middle;dominant-baseline:middle;font-family:{self.font}\">"
-                f"{self.number}</text>\n")
+        return (
+            f"<circle cx='{c[0]}' cy='{c[1]}' r='{self.radius}' "
+            f"stroke='{self.color}' stroke-width='{self.width}' fill='{self.fill_color}' />\n"
+            f"<text x='{c[0]}' y='{c[1]}' fill='{self.text_color}' font-size='{self.font_size}'"
+            f' style="text-anchor:middle;dominant-baseline:middle;font-family:{self.font}">'
+            f"{self.number}</text>\n"
+        )
 
-    def as_tikz(
-        self, map_pt: typing.Callable[[PointType], typing.Tuple[float, float]]
-    ) -> str:
+    def as_tikz(self, map_pt: typing.Callable[[PointType], typing.Tuple[float, float]]) -> str:
         """Return Tikz format.
 
         Args:
@@ -457,11 +506,13 @@ class NCircle(PictureElement):
             A Tikz string
         """
         c = map_pt(self.centre)
-        return (f"\\draw[{colors.get_tikz_name(self.color)},line width={self.width * 0.2}pt,"
-                f"fill={colors.get_tikz_name(self.fill_color)}] "
-                f"({c[0]},{c[1]}) circle ({self.radius * 0.2}pt);\n"
-                f"\\node[{colors.get_tikz_name(self.text_color)},anchor=center] "
-                f"at ({c[0]},{c[1]}) {{{tex_font_size(self.font_size)} {self.number}}};")
+        return (
+            f"\\draw[{self.colors.get_tikz_name(self.color)},line width={self.width * 0.2}pt,"
+            f"fill={self.colors.get_tikz_name(self.fill_color)}] "
+            f"({c[0]},{c[1]}) circle ({self.radius * 0.2}pt);\n"
+            f"\\node[{self.colors.get_tikz_name(self.text_color)},anchor=center] "
+            f"at ({c[0]},{c[1]}) {{{tex_font_size(self.font_size)} {self.number}}};\n"
+        )
 
     @property
     def points(self) -> SetOfPoints:
@@ -470,27 +521,27 @@ class NCircle(PictureElement):
         Returns:
             A set of points
         """
-        return (self.centre, )
+        return (self.centre,)
 
 
 class Fill(PictureElement):
     """A filled polygon."""
 
-    def __init__(self, vertices: SetOfPoints, color: str, opacity: float):
+    def __init__(self, colors: Colors, vertices: SetOfPoints, color: str, opacity: float):
         """Create a filled polygon.
 
         Args:
+            colors: Colors object for the current picture
             vertices: The vertices of the polygon in clockwise or anticlockwise order
             color: The colour to fill the polygon
             opacity: The opacity
         """
+        super().__init__(colors, tuple(sum(i) for i in zip(*vertices)))
         self.vertices = vertices
         self.color = color
         self.opacity = opacity
 
-    def as_svg(
-        self, map_pt: typing.Callable[[PointType], typing.Tuple[float, float]]
-    ) -> str:
+    def as_svg(self, map_pt: typing.Callable[[PointType], typing.Tuple[float, float]]) -> str:
         """Return SVG format.
 
         Args:
@@ -503,9 +554,7 @@ class Fill(PictureElement):
         ptstring = " ".join(f"{p[0]},{p[1]}" for p in pts)
         return f"<polygon points='{ptstring}' fill='{self.color}' opacity='{self.opacity}' />"
 
-    def as_tikz(
-        self, map_pt: typing.Callable[[PointType], typing.Tuple[float, float]]
-    ) -> str:
+    def as_tikz(self, map_pt: typing.Callable[[PointType], typing.Tuple[float, float]]) -> str:
         """Return Tikz format.
 
         Args:
@@ -515,8 +564,10 @@ class Fill(PictureElement):
             A Tikz string
         """
         vs = [map_pt(v) for v in self.vertices]
-        return (f"\\fill[{colors.get_tikz_name(self.color)},opacity={self.opacity}]"
-                " " + " -- ".join([f"({v[0]},{v[1]})" for v in vs]) + " -- cycle;")
+        return (
+            f"\\fill[{self.colors.get_tikz_name(self.color)},opacity={self.opacity}]"
+            " " + " -- ".join([f"({v[0]},{v[1]})" for v in vs]) + " -- cycle;\n"
+        )
 
     @property
     def points(self) -> SetOfPoints:
@@ -531,29 +582,38 @@ class Fill(PictureElement):
 class Math(PictureElement):
     """A mathematical symbol."""
 
-    def __init__(self, point: PointType, math: str, color: str, font_size: int,
-                 anchor: str):
+    def __init__(
+        self, colors: Colors, point: PointType, math: str, color: str, font_size: int, anchor: str
+    ):
         """Create a filled polygon.
 
         Args:
+            colors: Colors object for the current picture
             point: The point to put the math
             math: The math
             color: The color of the math
             font_size: The font size
             anchor: The point on the equation to anchor to
         """
+        super().__init__(colors, point)
         self.point = parse_point_input(point)
         self.math = math
         self.color = color
         self.font_size = font_size
         assert anchor in [
-            "center", "north", "south", "east", "west",
-            "north east", "north west", "south east", "south west"]
+            "center",
+            "north",
+            "south",
+            "east",
+            "west",
+            "north east",
+            "north west",
+            "south east",
+            "south west",
+        ]
         self.anchor = anchor
 
-    def as_svg(
-        self, map_pt: typing.Callable[[PointType], typing.Tuple[float, float]]
-    ) -> str:
+    def as_svg(self, map_pt: typing.Callable[[PointType], typing.Tuple[float, float]]) -> str:
         """Return SVG format.
 
         Args:
@@ -582,13 +642,13 @@ class Math(PictureElement):
 
         c = map_pt(self.point)
 
-        return (f"<text x='{c[0]}' y='{c[1]}' fill='{self.color}' font-size='{self.font_size}'"
-                f" style=\"{anchor};font-family:'CMU Serif',serif;font-style:italic\">"
-                f"{self.math}</text>\n")
+        return (
+            f"<text x='{c[0]}' y='{c[1]}' fill='{self.color}' font-size='{self.font_size}'"
+            f" style=\"{anchor};font-family:'CMU Serif',serif;font-style:italic\">"
+            f"{self.math}</text>\n"
+        )
 
-    def as_tikz(
-        self, map_pt: typing.Callable[[PointType], typing.Tuple[float, float]]
-    ) -> str:
+    def as_tikz(self, map_pt: typing.Callable[[PointType], typing.Tuple[float, float]]) -> str:
         """Return Tikz format.
 
         Args:
@@ -598,8 +658,10 @@ class Math(PictureElement):
             A Tikz string
         """
         p = map_pt(self.point)
-        return (f"\\node[{colors.get_tikz_name(self.color)},anchor={self.anchor}] "
-                f"at ({p[0]},{p[1]}) {{{tex_font_size(self.font_size)}${self.math}$}};")
+        return (
+            f"\\;node[{self.colors.get_tikz_name(self.color)},anchor={self.anchor}] "
+            f"at ({p[0]},{p[1]}) {{{tex_font_size(self.font_size)}${self.math}$}};\n"
+        )
 
     @property
     def points(self) -> SetOfPoints:
@@ -608,7 +670,7 @@ class Math(PictureElement):
         Returns:
             A set of points
         """
-        return (self.point, )
+        return (self.point,)
 
 
 class Picture:
@@ -617,10 +679,17 @@ class Picture:
     axes_3d: SetOfPoints
 
     def __init__(
-        self, padding: sympy.core.expr.Expr = sympy.Integer(25), scale: int = None,
-        width: int = None, height: int = None, axes_3d: SetOfPointsInput = None,
+        self,
+        padding: sympy.core.expr.Expr = sympy.Integer(25),
+        scale: typing.Optional[int] = None,
+        width: typing.Optional[int] = None,
+        height: typing.Optional[int] = None,
+        axes_3d: typing.Optional[SetOfPointsInput] = None,
         dof_arrow_size: typing.Union[int, sympy.core.expr.Expr] = 1,
-        title: str = None, desc: str = None, svg_metadata: str = None, tex_comment: str = None
+        title: typing.Optional[str] = None,
+        desc: typing.Optional[str] = None,
+        svg_metadata: typing.Optional[str] = None,
+        tex_comment: typing.Optional[str] = None,
     ):
         """Create a picture.
 
@@ -639,7 +708,8 @@ class Picture:
         self._default_axes: SetOfPoints = (
             (sympy.Integer(1), -sympy.Rational(2, 25)),
             (sympy.Rational(1, 2), sympy.Rational(1, 5)),
-            (sympy.Integer(0), sympy.Integer(1)))
+            (sympy.Integer(0), sympy.Integer(1)),
+        )
         self.elements: typing.List[PictureElement] = []
         self.padding = padding
         self.scale = scale
@@ -654,21 +724,25 @@ class Picture:
         self.desc = desc
         if svg_metadata is None:
             import symfem
+
             self.svg_metadata = (
                 "<!--\n"
                 "This diagram was created using Symfem\n"
                 f"{symfem.__github__}\n"
                 f"{symfem.__citation__}\n"
-                "-->\n")
+                "-->\n"
+            )
         else:
             assert isinstance(svg_metadata, str)
             self.svg_metadata = svg_metadata
         if tex_comment is None:
             import symfem
+
             self.tex_comment = (
                 "% This diagram was created using Symfem\n"
                 f"% {symfem.__github__}\n"
-                f"% {symfem.__citation__}\n\n")
+                f"% {symfem.__citation__}\n\n"
+            )
         else:
             assert isinstance(tex_comment, str)
             self.tex_comment = tex_comment
@@ -677,6 +751,7 @@ class Picture:
             self.axes_3d = self._default_axes
         else:
             self.axes_3d = parse_set_of_points_input(axes_3d)
+        self.colors = Colors()
 
     def z(self, p_in: PointOrFunction) -> sympy.core.expr.Expr:
         """Get the into/out-of-the-page component of a point.
@@ -712,7 +787,8 @@ class Picture:
         if len(p) == 3:
             return (
                 self.axes_3d[0][0] * p[0] + self.axes_3d[1][0] * p[1] + self.axes_3d[2][0] * p[2],
-                self.axes_3d[0][1] * p[0] + self.axes_3d[1][1] * p[1] + self.axes_3d[2][1] * p[2])
+                self.axes_3d[0][1] * p[0] + self.axes_3d[1][1] * p[1] + self.axes_3d[2][1] * p[2],
+            )
         raise ValueError(f"Unsupported gdim: {len(p)}")
 
     def parse_point(self, p: PointOrFunction) -> PointType:
@@ -724,7 +800,7 @@ class Picture:
         Returns:
             The point as a tuple of Sympy expressions
         """
-        if isinstance(p, AnyFunction):
+        if isinstance(p, Function):
             p_s = p.as_sympy()
             assert isinstance(p_s, tuple)
             p = p_s
@@ -732,63 +808,177 @@ class Picture:
         return self.to_2d(parse_point_input(p))
 
     def add_line(
-        self, start: PointOrFunction, end: PointOrFunction, color: str = colors.BLACK,
-        width: float = 4.0
+        self,
+        start: PointOrFunction,
+        end: PointOrFunction,
+        color: typing.Optional[str] = None,
+        width: float = 4.0,
     ):
-        """Add a line to the picture."""
-        self.elements.append(Line(self.parse_point(start), self.parse_point(end), color, width))
+        """Add a line to the picture.
+
+        Args:
+            start: The start point of the line
+            end: The end point of the line
+            color: The color of the line
+            width: The width of the line
+        """
+        self.elements.append(
+            Line(
+                self.colors,
+                self.parse_point(start),
+                self.parse_point(end),
+                self.colors.BLACK if color is None else color,
+                width,
+            )
+        )
 
     def add_bezier(
-        self, start: PointOrFunction, mid1: PointOrFunction, mid2: PointOrFunction,
-        end: PointOrFunction, color: str = colors.BLACK, width: float = 4.0
+        self,
+        start: PointOrFunction,
+        mid1: PointOrFunction,
+        mid2: PointOrFunction,
+        end: PointOrFunction,
+        color: typing.Optional[str] = None,
+        width: float = 4.0,
     ):
-        """Add a Bezier curve to the picture."""
-        self.elements.append(Bezier(
-            self.parse_point(start), self.parse_point(mid1), self.parse_point(mid2),
-            self.parse_point(end), color, width))
+        """Add a Bezier curve to the picture.
+
+        Args:
+            start: The start point of the Bezier curve
+            mid1: The first control point
+            mid2: The second control point
+            end: The end point of the Bezier curve
+            color: The color of the Bezier curve
+            width: The width of the Bezier curve
+        """
+        self.elements.append(
+            Bezier(
+                self.colors,
+                self.parse_point(start),
+                self.parse_point(mid1),
+                self.parse_point(mid2),
+                self.parse_point(end),
+                self.colors.BLACK if color is None else color,
+                width,
+            )
+        )
 
     def add_arrow(
-        self, start: PointOrFunction, end: PointOrFunction, color: str = colors.BLACK,
-        width: float = 4.0
+        self,
+        start: PointOrFunction,
+        end: PointOrFunction,
+        color: typing.Optional[str] = None,
+        width: float = 4.0,
     ):
-        """Add an arrow to the picture."""
-        self.elements.append(Arrow(self.parse_point(start), self.parse_point(end), color, width))
+        """Add an arrow to the picture.
 
-    def add_dof_marker(
-        self, point: PointOrFunction, number: int, color: str, bold: bool = True
-    ):
-        """Add a DOF marker."""
+        Args:
+            start: The start point of the arrow
+            end: The end point of the arrow
+            color: The color of the arrow
+            width: The width of the arrow
+        """
+        self.elements.append(
+            Arrow(
+                self.colors,
+                self.parse_point(start),
+                self.parse_point(end),
+                self.colors.BLACK if color is None else color,
+                width,
+            )
+        )
+
+    def add_dof_marker(self, point: PointOrFunction, number: int, color: str, bold: bool = True):
+        """Add a DOF marker.
+
+        Args:
+            point: The point
+            number: The number to put in the marker
+            color: The color of the marker
+            bold: Should the marker be bold?
+        """
         if bold:
-            self.add_ncircle(point, number, colors.BLACK, colors.BLACK, color)
+            self.add_ncircle(point, number, self.colors.BLACK, self.colors.BLACK, color)
         else:
-            self.add_ncircle(point, number, color, color, colors.WHITE)
+            self.add_ncircle(point, number, color, color, self.colors.WHITE)
 
     def add_dof_arrow(
-        self, point: PointOrFunction, direction: PointOrFunction, number: int,
-        color: str = colors.PURPLE, shifted: bool = False, bold: bool = True
+        self,
+        point: PointOrFunction,
+        direction: PointOrFunction,
+        number: int,
+        color: typing.Optional[str] = None,
+        shifted: bool = False,
+        bold: bool = True,
     ):
-        """Add a DOF arrow."""
+        """Add a DOF arrow.
+
+        Args:
+            point: The point
+            direction: The direction of the arrow
+            number: The number to put in the marker
+            color: The color of the marker
+            shifted: Should the marker be shifted?
+            bold: Should the marker be bold?
+        """
         vdirection = VectorFunction(self.parse_point(direction))
         vdirection /= 8 * vdirection.norm()
         vdirection *= self.dof_arrow_size
         start = VectorFunction(self.parse_point(point))
         if shifted:
             start += vdirection / 3
-        self.add_arrow(start, start + vdirection, color)
-        self.add_dof_marker(start, number, color, bold)
+        c = self.colors.PURPLE if color is None else color
+        self.add_arrow(start, start + vdirection, c)
+        self.add_dof_marker(start, number, c, bold)
 
     def add_ncircle(
-        self, centre: PointOrFunction, number: int, color: str = "red",
-        text_color: str = colors.BLACK, fill_color: str = colors.WHITE, radius: float = 20.0,
-        font_size: int = None, width: float = 4.0, font: str = "'Varela Round',sans-serif"
+        self,
+        centre: PointOrFunction,
+        number: int,
+        color: typing.Optional[str] = None,
+        text_color: typing.Optional[str] = None,
+        fill_color: typing.Optional[str] = None,
+        radius: float = 20.0,
+        font_size: typing.Optional[int] = None,
+        width: float = 4.0,
+        font: str = "'Varela Round',sans-serif",
     ):
-        """Add a numbered circle to the picture."""
-        self.elements.append(NCircle(
-            self.parse_point(centre), number, color, text_color, fill_color, radius, font_size,
-            width, font))
+        """Add a numbered circle to the picture.
 
-    def add_math(self, point: PointTypeInput, math: str, color: str = colors.BLACK,
-                 font_size: int = 35, anchor="center"):
+        Args:
+            centre: The centre points
+            number: The number in the circle
+            color: The color of the outline
+            text_color: The color of the test
+            fill_color: The colour of the background fill
+            radius: The radius of the circle
+            font_size: The font size
+            width: The width of the line
+            font: The font
+        """
+        self.elements.append(
+            NCircle(
+                self.colors,
+                self.parse_point(centre),
+                number,
+                self.colors.RED if color is None else color,
+                self.colors.BLACK if text_color is None else text_color,
+                self.colors.WHITE if fill_color is None else fill_color,
+                radius,
+                font_size,
+                width,
+                font,
+            )
+        )
+
+    def add_math(
+        self,
+        point: PointTypeInput,
+        math: str,
+        color: typing.Optional[str] = None,
+        font_size: int = 35,
+        anchor="center",
+    ):
         """Create mathematical symbol.
 
         Args:
@@ -798,19 +988,56 @@ class Picture:
             font_size: The font size
             anchor: The point on the equation to anchor to
         """
-        self.elements.append(Math(self.parse_point(point), math, color, font_size, anchor))
+        self.elements.append(
+            Math(
+                self.colors,
+                self.parse_point(point),
+                math,
+                self.colors.BLACK if color is None else color,
+                font_size,
+                anchor,
+            )
+        )
 
     def add_fill(
-        self, vertices: SetOfPointsOrFunctions, color: str = "red", opacity: float = 1.0
+        self,
+        vertices: SetOfPointsOrFunctions,
+        color: typing.Optional[str] = None,
+        opacity: float = 1.0,
     ):
-        """Add a filled polygon to the picture."""
-        self.elements.append(Fill(tuple(self.parse_point(p) for p in vertices), color, opacity))
+        """Add a filled polygon to the picture.
 
-    def compute_scale(self, unit: str = "px", reverse_y: bool = True) -> typing.Tuple[
-        sympy.core.expr.Expr, sympy.core.expr.Expr, sympy.core.expr.Expr,
-        typing.Callable[[PointType], typing.Tuple[float, float]]
+        Args:
+            vertices: The vertices of the polygon
+            color: The color of the polygon
+            opacity: The opacity of the polygon
+        """
+        self.elements.append(
+            Fill(
+                self.colors,
+                tuple(self.parse_point(p) for p in vertices),
+                self.colors.RED if color is None else color,
+                opacity,
+            )
+        )
+
+    def compute_scale(
+        self, unit: str = "px", reverse_y: bool = True
+    ) -> typing.Tuple[
+        sympy.core.expr.Expr,
+        sympy.core.expr.Expr,
+        sympy.core.expr.Expr,
+        typing.Callable[[PointType], typing.Tuple[float, float]],
     ]:
-        """Compute the scale and size of the picture."""
+        """Compute the scale and size of the picture.
+
+        Args:
+            unit: The unit to use. Accepted values: px, cm, mm
+            reverse_y: Should the y-axis be reversed?
+
+        Returns:
+            The scale, height, and width of the image, and a mapping function
+        """
         minx = min(i.minx() for i in self.elements)
         miny = min(i.miny() for i in self.elements)
         maxx = max(i.maxx() for i in self.elements)
@@ -841,27 +1068,40 @@ class Picture:
         height = 2 * self.padding + (maxy - miny) * scale
 
         if reverse_y:
+
             def map_pt(pt: PointType) -> typing.Tuple[float, float]:
                 """Map a point."""
                 return (
                     float(self.padding + (pt[0] - minx) * scale),
-                    float(height - self.padding - (pt[1] - miny) * scale))
+                    float(height - self.padding - (pt[1] - miny) * scale),
+                )
         else:
+
             def map_pt(pt: PointType) -> typing.Tuple[float, float]:
                 """Map a point."""
                 return (
                     float(self.padding + (pt[0] - minx) * scale),
-                    float(self.padding + (pt[1] - miny) * scale))
+                    float(self.padding + (pt[1] - miny) * scale),
+                )
 
         return scale, height, width, map_pt
 
-    def as_svg(self, filename: str = None) -> str:
-        """Convert to an SVG."""
+    def as_svg(self, filename: typing.Optional[str] = None) -> str:
+        """Convert to an SVG.
+
+        Args:
+            filename: The file name
+
+        Returns:
+            The image as an SVG string
+        """
         scale, height, width, map_pt = self.compute_scale("px")
 
         assert filename is None or filename.endswith(".svg")
-        img = (f"<svg width='{float(width)}' height='{float(height)}'"
-               " xmlns='http://www.w3.org/2000/svg' xmlns:xlink='http://www.w3.org/1999/xlink'>\n")
+        img = (
+            f"<svg width='{float(width)}' height='{float(height)}'"
+            " xmlns='http://www.w3.org/2000/svg' xmlns:xlink='http://www.w3.org/1999/xlink'>\n"
+        )
         if self.title is not None:
             img += f"<title>{self.title}</title>\n"
         if self.desc is not None:
@@ -869,6 +1109,8 @@ class Picture:
         if self.svg_metadata is not None:
             img += self.svg_metadata + "\n"
 
+        self.elements.sort(key=lambda e: self.z(e.midpoint))
+
         for e in self.elements:
             img += e.as_svg(map_pt)
 
@@ -880,14 +1122,25 @@ class Picture:
 
         return img
 
-    def as_png(self, filename: str, png_scale: float = None, png_width: int = None,
-               png_height: int = None):
-        """Convert to a PNG."""
+    def as_png(
+        self,
+        filename: str,
+        png_scale: typing.Optional[float] = None,
+        png_width: typing.Optional[int] = None,
+        png_height: typing.Optional[int] = None,
+    ):
+        """Convert to a PNG.
+
+        Args:
+            filename: The file name
+            png_scale: The scale of the png
+            png_width: The width of the png
+            png_height: The height of the png
+        """
         try:
             from cairosvg import svg2png
         except ImportError:
-            raise ImportError("CairoSVG is needed for plotting PNGs"
-                              " (pip install CairoSVG)")
+            raise ImportError("CairoSVG is needed for plotting PNGs (pip install CairoSVG)")
 
         if png_scale is not None:
             assert png_width is None
@@ -902,10 +1155,17 @@ class Picture:
 
         assert isinstance(png_scale, float)
         assert filename.endswith(".png")
-        svg2png(bytestring=self.as_svg(), write_to=filename, scale=png_scale)
+        svg2png(bytestring=self.as_svg().encode(), write_to=filename, scale=png_scale)
 
-    def as_tikz(self, filename: str = None) -> str:
-        """Convert to tikz."""
+    def as_tikz(self, filename: typing.Optional[str] = None) -> str:
+        """Convert to tikz.
+
+        Args:
+            filename: The file name
+
+        Returns:
+            The image as a Tikz string
+        """
         scale, height, width, map_pt = self.compute_scale("cm", False)
         tikz = self.tex_comment
         tikz += "\\begin{tikzpicture}[x=1cm,y=1cm]\n"
@@ -914,7 +1174,7 @@ class Picture:
         for e in self.elements:
             inner_tikz += e.as_tikz(map_pt)
 
-        tikz += colors.get_tikz_definitions() + inner_tikz
+        tikz += self.colors.get_tikz_definitions() + inner_tikz
 
         tikz += "\\end{tikzpicture}\n"
 
@@ -924,9 +1184,17 @@ class Picture:
 
         return tikz
 
-    def save(self, filename: typing.Union[str, typing.List[str]],
-             plot_options: typing.Dict[str, typing.Any] = {}):
-        """Save the picture as a file."""
+    def save(
+        self,
+        filename: typing.Union[str, typing.List[str]],
+        plot_options: typing.Dict[str, typing.Any] = {},
+    ):
+        """Save the picture as a file.
+
+        Args:
+            filename: The file name
+            plot_options: The plotting options
+        """
         if isinstance(filename, list):
             for fn in filename:
                 self.save(fn, plot_options)
