@@ -4,7 +4,9 @@ import typing
 
 import sympy
 
-from .functions import ScalarFunction
+from symfem.functions import ScalarFunction
+
+__all__ = ["allequal"]
 
 
 def allequal(a: typing.Any, b: typing.Any) -> bool:
@@ -25,5 +27,5 @@ def allequal(a: typing.Any, b: typing.Any) -> bool:
     if isinstance(a, sympy.core.expr.Expr):
         a = ScalarFunction(a)
     if isinstance(b, sympy.core.expr.Expr):
-        a = ScalarFunction(b)
+        b = ScalarFunction(b)
     return a == b
